<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification; 

class Fai32000DespesasController extends \Teknisa\Libs\Controller\CRUD{
    protected $dataSourceName = "proddespprev";
    protected $dataSourceNameAccount = "ctbldespprev";
    protected $dataSourceManager;
    protected $connection;
    protected $newCode;
    protected $environment;

    
    public function __construct($crudService, $environment, $dataSourceManager, $connection, $newCode) {
        $this->dataSourceManager    = $dataSourceManager;
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->environment = $environment;
        parent::__construct($crudService, $environment);
        
    }
    
    public function excluiDespesa(DTO\Request $request, DTO\Response $response ){
        
        $row = $request->getParameter("DataSet");
        
        if($row != null){
            $despesa = $row[0];
            
            try{
                $this->connection->beginTransaction();
                $filter = array(
                    array(
                        "columnName" => "CDDESPPREV",
                        "operator" => "=",
                        "value" => $despesa["CDDESPPREV"]
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator" => "=",
                        "value" => $despesa["NRORG"]
                    )
                );
                $dataSetProd = $this->crudService->getFilteredDataSet($this->dataSourceName, $filter);
    		    $this->dataSourceManager->delete($dataSetProd);
                
                $dataSetAccount = $this->crudService->getFilteredDataSet($this->dataSourceNameAccount, $filter);
    		    $this->dataSourceManager->delete($dataSetAccount);
    		    
    		    $dataSet = $this->crudService->getFilteredDataSet('despprev', $filter);
                $this->dataSourceManager->delete($dataSet);
                
                $this->connection->commit();
                
            }catch(\Exception $ex){
                if(strpos($ex->getMessage(), "ORA-02292") === FALSE){
                    $response->setError(new Error("FALHA_PROC_DADOS " . $ex->getMessage(), 500));
                }else{
                    $response->setError(new Error("DESPREGISTROVINCULADO", 500));
                }
                
                $this->connection->rollback();
            }

        }
    }

    public function saveProduto(DTO\Request $request, DTO\Response $response ){

        $rows = $request->getDataSet()->getRows();
        $erroList = array();
        $saved = false;
        if($rows != null){
            try{
                $rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
                foreach($rowsToSave as $row){
                    try{
                        $validProd = $this->validaProduto($row["CDPRODUTO"], $response);
                        if(empty($validProd)){
                            $this->crudService->save($this->dataSourceName, [$row]);
                            $saved = true;
                        }
                        else{
                            foreach($validProd as $prod){
                                array_push($erroList, $prod);
                            }
                        }
                    }catch(\Exception $ex){
                        if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                            array_push($erroList, "FALHA_PROC_DADOS " . $ex->getMessage());
                        }else{
                            array_push($erroList, $row["NMPRODUTO"]);
                        }    
                    }
                }
            }catch(\Exception $ex){
                if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                    array_push($erroList, "FALHA_PROC_DADOS " . $ex->getMessage());
                }else{
                    array_push($erroList, $row["NMPRODUTO"]);
                }
            }
            
            if(!empty($erroList)){
                $response->addDataSet(new DataSet("errorList", $erroList));
                $response->addDataSet(new DataSet("saved", [$saved]));
                $response->setError(new Error("Ocorreram erros durante a operação. Verifique o log", 500));
            }
        } 
    }
    
    public function saveDespesa(DTO\Request $request, DTO\Response $response ){
        $row = $request->getRow();
        $dataSourceName = "despprev";

        $row["IDDESTACAROP"] = "";
        
        if($row["__is_new"]){
            if(isset($row['CDPRODUTO'])){
                $this->saveDespesaProduto($row, $dataSourceName, $response);
            } 
            
            if(isset($row['CDCONTCTBL'])){
                $this->saveDespesaCont($row, $dataSourceName, $response);
            } 
        }else{
            $this->updateDespesa($row, $dataSourceName, $response);
        }
    }
    
    private function saveDespesaCont($row, $dataSourceName, $response) {
        try{
            $qtde_account_insert = 0;
            $qtde_account = 0;
            $qtde_account = count($row["CDCONTCTBL"]);
            $accountDesp = array();
            $errorList = array();
            
            foreach($row["CDCONTCTBL"] as $account){
                $accountInsert["NRCTBLDESPPREV"] = $this->newCode->getCodeByTableName("CTBLDESPPREV", $this->environment->getNrOrg());
                $accountInsert["CDCONTCTBL"] = $account;
                $accountInsert["CDDESPPREV"] = $row["CDDESPPREV"];
                $accountInsert["__is_new"] = true;
                
                $accountDuplicated = $this->validaConta($account, $response);
                if(empty($accountDuplicated)){
                    array_push($accountDesp, $accountInsert);
                }else{
                    foreach($accountDuplicated as $accountErr){
                        array_push($errorList, $accountErr);
                    }
                }
            }
            if(!empty($accountDesp)){
                $dataSet = new DataSet($dataSourceName, [$row]);
                $request = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
                $this->crudService->save($dataSourceName, $rowToSave);
    
                $dataSetAccount = new DataSet($this->dataSourceNameAccount, $accountDesp);
                $requestAccount = new DTO\Request\DataSet($dataSetAccount, "", "", "");
                $rowToSaveAccount = $this->crudService->getRowsToSave($requestAccount, $this->dataSourceNameAccount);
                $this->crudService->save($this->dataSourceNameAccount, $rowToSaveAccount);
                
                $qtde_account_insert = count($accountDesp);
                $response->addDataSet(new DataSet("ACCOUNTINSERT", [$qtde_account == $qtde_account_insert]));
                $response->addDataSet(new DataSet("primaryKey", array("CDDESPPREV"=>$row["CDDESPPREV"])));
                
            }else{
                array_push($errorList, "INFO_UMA_CONTA_VALIDA");
            }
        }catch(\Exception $ex){
            
            if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                array_push($errorList, "FALHA_PROC_DADOS " . $ex->getMessage());
            }else{
                array_push($errorList, "DESPJACAD");
            }
        }
        
        if(!empty($errorList) && !(($qtde_account - $qtde_account_insert) == 0)){
            $response->addDataSet(new DataSet("errorList", $errorList));
             $response->setError(new Error("Ocorreram erros durante a operação. Verifique o log", 500));
        }   
    
    } 
    
    private function saveDespesaProduto($row, $dataSourceName, $response) {
        try{
            $qtde_prod_insert = 0;
            $qtde_prod = 0;
            $qtde_prod = count($row["CDPRODUTO"]);
            $prod = array();
            $errorList = array();
            
            foreach($row["CDPRODUTO"] as $produto){
                $produtoInsert["NRPRODDESPPREV"] = $this->newCode->getCodeByTableName("PRODDESPPREV", $this->environment->getNrOrg());
                $produtoInsert["CDPRODUTO"] = $produto;
                $produtoInsert["CDDESPPREV"] = $row["CDDESPPREV"];
                $produtoInsert["__is_new"] = true;
                
                $valid = $this->validaProduto($produto, $response);
                if(empty($valid)){
                    array_push($prod, $produtoInsert);
                }else{
                    foreach($valid as $prodErr){
                        array_push($errorList, $prodErr);
                    }
                }
            }
            if(!empty($prod)){
                $dataSet = new DataSet($dataSourceName, [$row]);
                $request = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
                $this->crudService->save($dataSourceName, $rowToSave);
    
                $dataSetProd = new DataSet($this->dataSourceName, $prod);
                $requestProd = new DTO\Request\DataSet($dataSetProd, "", "", "");
                $rowToSaveProduto = $this->crudService->getRowsToSave($requestProd, $this->dataSourceName);
                $this->crudService->save($this->dataSourceName, $rowToSaveProduto);
                
                $qtde_prod_insert = count($prod);
                $response->addDataSet(new DataSet("PRODINSERT", [$qtde_prod == $qtde_prod_insert]));
                $response->addDataSet(new DataSet("primaryKey", array("CDDESPPREV"=>$row["CDDESPPREV"])));
                
            }else{
                array_push($errorList, "INFO_UM_PROD_VALID");
            }
        }catch(\Exception $ex){
            
            if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                array_push($errorList, "FALHA_PROC_DADOS " . $ex->getMessage());
            }else{
                array_push($errorList, "DESPJACAD");
            }
        }
        
        if(!empty($errorList) && !(($qtde_prod - $qtde_prod_insert) == 0)){
            $response->addDataSet(new DataSet("errorList", $errorList));
            $response->setError(new Error("Ocorreram erros durante a operação. Verifique o log", 500));
        }   
    }
    
    private function updateDespesa ($row, $dataSourceName, $response) {
        try {
            $errorList = array();
            $tipo = '';
            if($row['IDTIPODESPESA'] == 'ENTRADA'){
                $tipo = 'LANCENTRADA';
            }else{
                $tipo = $row['IDTIPODESPESA'];
            }
       
            $params =  array(
                    'P_CDDESPPREV' => $row['CDDESPPREV'],
                    'P_IDTIPODESPESA' => $tipo
                );
            $stmt = $this->connection->prepare("SELECT COUNT(*) RESULTADO
                                                  FROM (SELECT CDDESPPREV FROM PERCITEMDETALGOP 
                                                         WHERE CDDESPPREV = :P_CDDESPPREV
                                                           AND IDDESPPREV <> :P_IDTIPODESPESA
                                                       UNION
                                                        SELECT CDDESPPREV FROM PERCITDETALFILGOP 
                                                         WHERE CDDESPPREV = :P_CDDESPPREV
                                                           AND IDDESPPREV <> :P_IDTIPODESPESA)
                                                    ");
            $stmt->execute($params);
            $result = $stmt->fetchAll();
    	    if($row["FLAG"]){
    	        $dataSet = new DataSet($dataSourceName, [$row]);
                $request = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
                $this->crudService->save($dataSourceName, $rowToSave);
    	    }
    	    
            $response->addDataSet(new DataSet("data", $result));
            
        }catch(\Exception $ex){
            
            if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                array_push($errorList, "FALHA_PROC_DADOS " . $ex->getMessage());
            }else{
                array_push($errorList, "DESPJACAD");
            }
        }
        
         if(!empty($errorList)){
            $response->addDataSet(new DataSet("errorList", $errorList));
            $response->setError(new Error("Ocorreram erros durante a operação. Verifique o log", 500));
        } 
        
    }
    
    
    function maskApply($mask, $cod){
        
        $mask[0] = '*';
        $mask = str_replace("9", "*", $mask);
    
        for($i=0;$i<strlen($cod);$i++){
            $mask[strpos($mask,"*")] = $cod[$i];
        }
        if(strpos($mask, '*')){
            $mask= substr_replace($mask, '', strpos($mask, '.*'));
        }
        return $mask;
    }
    
    public function validaConta($cdConCtbl, $response) {
        
        $duplicatedAccount = array();
        $params =  array(
                            'P_CDCONTCTBL' => $cdConCtbl
                        );
        $stmt = $this->connection->prepare("SELECT C.CDCONTCTBL, C.NMCONTCTBL, CD.CDDESPPREV, D.NMDESPPREV
                                            FROM CTBLDESPPREV CD INNER JOIN CONTCTBL C ON C.CDCONTCTBL = CD.CDCONTCTBL
                                                                 INNER JOIN DESPPREV D ON D.CDDESPPREV = CD.CDDESPPREV
                                            WHERE CD.CDCONTCTBL LIKE (SELECT T.CDCONTCTBL
                                                                            FROM CONTCTBL T
                                                                            WHERE T.CDCONTCTBL = :P_CDCONTCTBL)||'%'
                                            ORDER BY CD.CDDESPPREV, C.CDCONTCTBL"); 
        $stmt->execute($params);
        $result = $stmt->fetchAll();
        
        if(!empty($result)){
            
            try{
                $params = array('NRORG' => $this->environment->getNrorg());
                $stmtMask = $this->connection->prepare("SELECT CDPICTCCTB MASK FROM PARAMGERAL WHERE NRORG = :NRORG"); 
                $stmtMask->execute($params);
    
                $mask = $stmtMask->fetchAll()[0]['MASK'];
                
                foreach($result as $account){
                    $account['CDCONTCTBL'] = $this->maskApply($mask, $account['CDCONTCTBL']);
                    
                    array_push($duplicatedAccount, array('CDCONTCTBL' => $account['CDCONTCTBL'],
                                                      'NMCONTCTBL' => $account['NMCONTCTBL'],
                                                      'CDDESPPREV' => $account['CDDESPPREV']
                                                      ));
                }
            }catch(\Exception $ex){
                $response->setError(new Error("FALHA_PROC_DADOS " . $ex->getMessage(), 500));
            }
        }
        return $duplicatedAccount;
    
    }
    
    public function validaProduto($cdProduto, $response){
        $duplicatedProd = array();
        $parametros =  array(
                            'P_CDPRODUTO' => $cdProduto
                        );
        $stmt = $this->connection->prepare("
          SELECT P.CDARVPROD, P.NMPRODUTO, PR.CDPRODUTO, PR.CDDESPPREV, D.NMDESPPREV 
            FROM PRODUTO P, PRODDESPPREV PR, DESPPREV D
           WHERE P.CDPRODUTO  = PR.CDPRODUTO
             AND P.CDARVPROD LIKE (SELECT P.CDARVPROD
                                     FROM PRODUTO P
                                    WHERE P.CDPRODUTO = :P_CDPRODUTO)||'%'
             AND PR.CDDESPPREV = D.CDDESPPREV"); 
        $stmt->execute($parametros);
        
        $result = $stmt->fetchAll();
        
        if(!empty($result)){
            
            try{
                $params = array('NRORG' => $this->environment->getNrorg());
                $stmtMask = $this->connection->prepare("SELECT CDPICTPROD MASK FROM PARAMGERAL WHERE NRORG = :NRORG"); 
                $stmtMask->execute($params);
    
                $mask = $stmtMask->fetchAll()[0]['MASK'];
                
                foreach($result as $produto){
                    $produto['CDARVPROD'] = $this->maskApply($mask, $produto['CDARVPROD']);
                    
                    array_push($duplicatedProd, array('CDARVPROD' => $produto['CDARVPROD'],
                                                      'NMPRODUTO' => $produto['NMPRODUTO'],
                                                      'CDDESPPREV' => $produto['CDDESPPREV']
                                                      ));
                }
            }catch(\Exception $ex){
                $response->setError(new Error("FALHA_PROC_DADOS " . $ex->getMessage(), 500));
            }
        }
        return $duplicatedProd;
    }
    
    public function verificaItem(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getDataSet()->getRows();
        $resp = 0;
        foreach($rows as $row){
            $parametros =  array(
                            'P_CDDESPPREV' => $rows[0]['CDDESPPREV']
                        );
            $stmt = $this->connection->prepare("SELECT MAX(RESULTADO.GERAL) EXISTE 
                                                    FROM (SELECT COUNT(1) AS GERAL 
                                                            FROM PERCITEMDETALGOP IG 
                                                           WHERE IG.CDDESPPREV = :P_CDDESPPREV
                                               UNION ALL
                                                SELECT COUNT(1) AS GERAL 
                                                    FROM PERCITDETALFILGOP I 
                                                 WHERE I.CDDESPPREV = :P_CDDESPPREV) 
                                                                            RESULTADO"); 
            $stmt->execute($parametros);
            $retorno = $stmt->fetchAll(); 
    	    $return[1] = $retorno[0]["EXISTE"];
    	    if($return[1]>0){
    		    $resp = 1;        
    	    }
        }
        if($resp != 0){
            $response->addDataSet(new DataSet("RESULTADO", $return)); 
        }else{
            $return[1] = 0;
            $response->addDataSet(new DataSet("RESULTADO", $return));
        }
    }
    
    public function excluirItemPercentual(DTO\Request $request, DTO\Response $response ){
       $rows = $request->getDataSet()->getRows();
       if(!empty($rows)){
           foreach ($rows as $row) {
               $parametros =  array(
                            'P_CDDESPPREV' => $row['CDDESPPREV'],
                            'P_CDPRODUTO'  => $row['CDPRODUTO']
                        );
           
               $stmt = $this->connection->prepare("SELECT PG.NRPERCITEMDETALGOP, PG.CDPRODUTO
                                                   FROM PERCDETALPRODGOP PG,PERCITEMDETALGOP IG 
                                                 WHERE IG.CDDESPPREV = :P_CDDESPPREV
                                                   AND PG.CDPRODUTO = :P_CDPRODUTO  
                                                   AND IG.NRPERCITEMDETALGOP = PG.NRPERCITEMDETALGOP
                                              ");
                                              
                $stmtUnid = $this->connection->prepare("SELECT PU.NRPERCITDETALFILGOP, PU.CDPRODUTO  
                                                       FROM PERCDETPRODFILGOP PU,PERCITDETALFILGOP IU 
                                                    WHERE IU.CDDESPPREV = :P_CDDESPPREV 
                                                      AND PU.CDPRODUTO = :P_CDPRODUTO
                                                      AND IU.NRPERCITDETALFILGOP = PU.NRPERCITDETALFILGOP
                                              ");
                $stmt->execute($parametros);
                $retorno = $stmt->fetchAll();
                $stmtUnid->execute($parametros);
                $retornoUnid = $stmtUnid->fetchAll();
                
                if(!empty($retorno[0]["NRPERCITEMDETALGOP"])){
                    $filtro = array(
                        array(
                            "columnName" => "NRPERCITEMDETALGOP",
                            "operator" => "=",
                            "value" => $retorno[0]["NRPERCITEMDETALGOP"]
                        ),
                        array(
                            "columnName" => "CDPRODUTO",
                            "operator" => "=",
                            "value" => $retorno[0]["CDPRODUTO"]
                        )
                    );
                    $dataSetProd = $this->crudService->getFilteredDataSet('percdetalprodgop', $filtro);
                    $this->dataSourceManager->delete($dataSetProd);
                }
                if(!empty($retornoUnid[0]["NRPERCITDETALFILGOP"])){
                    $filtro = array(
                        array(
                            "columnName" => "NRPERCITDETALFILGOP",
                            "operator" => "=",
                            "value" => $retornoUnid[0]["NRPERCITDETALFILGOP"]
                        ),
                        array(
                            "columnName" => "CDPRODUTO",
                            "operator" => "=",
                            "value" => $retorno[0]["CDPRODUTO"]
                        )
                    );
                    $dataSetProd = $this->crudService->getFilteredDataSet('percdetprodfilgop', $filtro);
                    $this->dataSourceManager->delete($dataSetProd);
                }
           }
       }
        
    }
    
    public function saveAccounts(DTO\Request $request, DTO\Response $response ){

        $rows = $request->getDataSet()->getRows();
        $errorList = array();
        $saved = false;
        if($rows != null){
            try{
                $rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceNameAccount);
                foreach($rowsToSave as $row){
                    try{
                        $AccountDuplicated = $this->validaConta($row["CDCONTCTBL"], $response);
                        if(empty($AccountDuplicated)){
                            $this->crudService->save($this->dataSourceNameAccount, [$row]);
                            $saved = true;
                        }
                        else{
                            foreach($AccountDuplicated as $accountErr){
                                array_push($errorList, $accountErr);
                            }
                        }
                    }catch(\Exception $ex){
                        if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                            array_push($errorList, "FALHA_PROC_DADOS " . $ex->getMessage());
                        }else{
                            array_push($errorList, $row["NMPRODUTO"]);
                        }    
                    }
                }
            }catch(\Exception $ex){
                if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                    array_push($errorList, "FALHA_PROC_DADOS " . $ex->getMessage());
                }else{
                    array_push($errorList, $row["NMPRODUTO"]);
                }
            }
            
            if(!empty($errorList)){
                $response->addDataSet(new DataSet("errorList", $errorList));
                $response->addDataSet(new DataSet("saved", [$saved]));
                $response->setError(new Error("Ocorreram erros durante a operação. Verifique o log", 500));
            }
        } 
    }
    
     public function deleteItemPercentAccount(DTO\Request $request, DTO\Response $response ){
       $rows = $request->getDataSet()->getRows();
       if(!empty($rows)){
           foreach ($rows as $row) {
               $parametros =  array(
                            'P_CDDESPPREV' => $row['CDDESPPREV'],
                            'P_CDCONTCTBL'  => $row['CDCONTCTBL']
                        );
           
               $stmt = $this->connection->prepare("SELECT PG.NRPERCITEMDETALGOP, PG.CDCONTCTBL
                                                   FROM PERCDETALPRODGOP PG,PERCITEMDETALGOP IG 
                                                 WHERE IG.CDDESPPREV = :P_CDDESPPREV
                                                   AND PG.CDPRODUTO = :P_CDCONTCTBL  
                                                   AND IG.NRPERCITEMDETALGOP = PG.NRPERCITEMDETALGOP
                                              ");
                                              
                $stmtUnid = $this->connection->prepare("SELECT PU.NRPERCITDETALFILGOP, PU.CDCONTCTBL  
                                                       FROM PERCDETPRODFILGOP PU,PERCITDETALFILGOP IU 
                                                    WHERE IU.CDDESPPREV = :P_CDDESPPREV 
                                                      AND PU.CDPRODUTO = :P_CDCONTCTBL
                                                      AND IU.NRPERCITDETALFILGOP = PU.NRPERCITDETALFILGOP
                                              ");
                $stmt->execute($parametros);
                $retorno = $stmt->fetchAll();
                $stmtUnid->execute($parametros);
                $retornoUnid = $stmtUnid->fetchAll();
                
                if(!empty($retorno[0]["NRPERCITEMDETALGOP"])){
                    $filtro = array(
                        array(
                            "columnName" => "NRPERCITEMDETALGOP",
                            "operator" => "=",
                            "value" => $retorno[0]["NRPERCITEMDETALGOP"]
                        ),
                        array(
                            "columnName" => "CDCONTCTBL",
                            "operator" => "=",
                            "value" => $retorno[0]["CDCONTCTBL"]
                        )
                    );
                    $dataSetAccount = $this->crudService->getFilteredDataSet('percdetalprodgop', $filtro);
                    $this->dataSourceManager->delete($dataSetProd);
                }
                if(!empty($retornoUnid[0]["NRPERCITDETALFILGOP"])){
                    $filtro = array(
                        array(
                            "columnName" => "NRPERCITDETALFILGOP",
                            "operator" => "=",
                            "value" => $retornoUnid[0]["NRPERCITDETALFILGOP"]
                        ),
                        array(
                            "columnName" => "CDCONTCTBL",
                            "operator" => "=",
                            "value" => $retorno[0]["CDCONTCTBL"]
                        )
                    );
                    $dataSetAccount = $this->crudService->getFilteredDataSet('percdetprodfilgop', $filtro);
                    $this->dataSourceManager->delete($dataSetProd);
                }
           }
       }
    }

}

